@@ -463,9 +463,9 @@ run.jags.study <- function(simulations, model, datafunction, targets=list(), con
 		
 		# Detect common problems:
 		if(!require("runjags") || package_version(utils::packageDescription('runjags', fields='Version'))<1)
-			stop(paste("The runjags package (version >=1.0.0) is not installed on the cluster node '", Sys.info()['nodename'], "'", sep="")) 
+			stop(paste("The runjags package (version >=1.0.0) is not installed (or failed to load) on the cluster node '", Sys.info()['nodename'], "'", sep="")) 
 		if(runjags.args$method%in%c('rjags','rjparallel') && !requireNamespace("rjags"))
-			stop(paste("The rjags package is not installed on the cluster node '", Sys.info()['nodename'], "' - try specifying method='simple'", sep="")) 
+			stop(paste("The rjags package is not installed (or failed to load) on the cluster node '", Sys.info()['nodename'], "' - try specifying method='simple'", sep="")) 
 
 		if(runjags.args$method%in%c('rjags','rjparallel')){
 			
