@@ -3,10 +3,14 @@
 	This file is Copyright (C) 2002-10 Martyn Plummer, 
 	from the source for JAGS version 3.3, licensed under GPL-2
 	
-	The only modification is the change in namespace to prevent clashes
+	The namespace has been modified to prevent clashes
+	This file is not required for JAGS version >= 4 and will be removed
+	from runjags once JAGS version 3 becomes obsolete
 
 */
 
+#ifdef INCLUDERSCALARDIST
+
 #include "PFunction.h"
 #include "RScalarDist.h"
 
@@ -42,3 +46,5 @@ namespace runjags {
 	return checkArgs(args);
     }
 }
+
+#endif  /* INCLUDERSCALARDIST */
